﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Apache.NMS;
using Apache.NMS.ActiveMQ;

namespace ConsoleCustomer
{
    class Program
    {
        static IConnectionFactory _factory = null;

        static void Main(string[] args)
        {
            try
            {
                //创建一个连接工厂
                _factory = new ConnectionFactory("tcp://localhost:61616/");

                using (IConnection conn = _factory.CreateConnection())
                {
                    //设置客户端ID
                    conn.ClientId = "Consumer";

                    //启动连接
                    conn.Start();

                    //创建会话
                    using (ISession session = conn.CreateSession())
                    {
                        //创建一个主题
                        var topic = new Apache.NMS.ActiveMQ.Commands.ActiveMQTopic("topic");

                        //创建一个持久的消费者/订阅者
                        IMessageConsumer consumer = session.CreateDurableConsumer(topic, "Consumer", null, false);

                        //绑定监听事件
                        consumer.Listener += new MessageListener(consumer_Listener);

                        //Read方法很重要，否则ISession将会被dispose
                        Console.Read();
                    }

                    //停止并关闭连接
                    conn.Stop();
                    conn.Close();
                }

            }
            catch (Exception ex)
            {
                Console.Write(ex.ToString());
            }

        }

        /// <summary>
        /// 监听事件
        /// </summary>
        /// <param name="message"></param>
        static void consumer_Listener(IMessage message)
        {
            ITextMessage msg = (ITextMessage)message;
            Console.WriteLine("Receive: " + msg.Text);  

        }
    }
}
