﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Apache.NMS;
using Apache.NMS.ActiveMQ;

namespace Consumer
{
    public partial class ConsumerFrm : Form
    {
        IConnectionFactory _factory = null;
        IConnection _conn = null;
        ISession _session = null;

        delegate void ChangeTextBoxValue(string str);

        public ConsumerFrm()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {

            base.OnLoad(e);
            btnSub.Click += new EventHandler(btnSub_Click);

        }

        /// <summary>
        /// 订阅按钮点击事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void btnSub_Click(object sender, EventArgs e)
        {
            Subscriber();
        }

        void Subscriber()
        {
            //这里创建Connection，Session不能使用using，否则会被dispose

            _factory = new ConnectionFactory("tcp://localhost:61616/");

            _conn = _factory.CreateConnection();

            _conn.ClientId = "Consumer";
            _conn.Start();

            _session = _conn.CreateSession();

            var topic = new Apache.NMS.ActiveMQ.Commands.ActiveMQTopic("topic");

            IMessageConsumer consumer = _session.CreateDurableConsumer(topic, "Consumer", null, false);

            consumer.Listener += new MessageListener(consumer_Listener);            
        }

        /// <summary>
        /// 监听器
        /// </summary>
        /// <param name="message"></param>
        void consumer_Listener(IMessage message)
        {
            //这里会涉及到子线程操作UI线程，所以得使用Invoke
            ITextMessage msg = (ITextMessage)message;
            SetText(msg.Text);
        }

        /// <summary>
        /// 设置UI信息
        /// </summary>
        /// <param name="str"></param>
        private void SetText(string str)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new ChangeTextBoxValue(SetText), str);
            }
            else
            {
                this.txtReceive.Text = str;
            }
        }

        /// <summary>
        /// 窗体关闭时释放资源
        /// </summary>
        /// <param name="e"></param>
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);
            _session.Dispose();
            _conn.Stop();
            _conn.Close();
        }
    }
}
